open Lexing

module SMap = Map.Make(String)
module SSet = Set.Make(String)

type pvar = string
type pchannel  = string

type var = string
type channel = string

type pprog =
  | PSend of pchannel * pprog
  | PRec  of pchannel * pvar * pprog
  | PComp of pprog * pprog
  | PExe  of pvar
  | PZero

type disp_send = DSend of disp_prog
and  disp_receive = DRec of var * disp_prog
and disp_prog = ((disp_send list) * (disp_receive list)) SMap.t * (var list)

type send = Send of prog
and  receive = Rec of (prog -> prog)

and prog = ((send list) * (receive list)) SMap.t * (var list)

let ($) = List.rev_append

let get_channel a p =
  SMap.find a (fst p)

let update_channel (a:channel) ((m,exe):prog) x =
  ((SMap.add a x m, exe):prog)

let is_empty_channel m =
  SMap.for_all (fun _ (s,r) -> s = [] && r = []) m

let channel_empty = SMap.empty
let channel_fold  = SMap.fold
let channel_iter  = SMap.iter
let channel_size  = SMap.cardinal

let in_chan_fold  = List.fold_left

let var_empty = []
let var_iter  = List.iter
let var_is_empty  x = x = []
let var_singleton x = [x]
let var_union = List.rev_append
let var_fold  = List.fold_left
let var_size  = List.length

let rec rev_concat l' = function
  | a::l -> rev_concat (a $ l') l
  | []   -> l'
