open Ast
open Parser
open Lexer
open Hammer
open Grinder

open Format
open Arg
open Lexing

let usage = "usage: compile file.hoc"

let file =
  let f = ref None in
  let set_file s =
    if not (Filename.check_suffix s ".hoc")
    then raise (Arg.Bad "no .hoc extension");
    f := Some s in
  Arg.parse [] set_file usage;
  match !f with
  | None -> Arg.usage [] usage; exit 1
  | Some f -> f

let reportPos (b,e) =
  let l = b.pos_lnum in
  let fc = b.pos_cnum - b.pos_bol + 1 in
  let lc = e.pos_cnum - b.pos_bol + 1 in
  eprintf "File \"%s\", line %d, characters %d-%d:\n" file l fc lc


let () =
  Random.self_init ();
  let c = open_in file in
  let lb = Lexing.from_channel c in
  try
    let f = Parser.init Lexer.token lb in
    close_in c;
    grind_compile f;
    match Unix.system "ocamlbuild main_compile.native" with
    | Unix.WEXITED 0 -> printf "Compilation succeeded@.";
      exit (Unix.execv "./main_compile.native" (Array.of_list []))
    | _ -> close_in c; eprintf "Compilation error@."; exit 1
  with
  | Lexer.LexerError s ->
    close_in c;
    reportPos  (lexeme_start_p lb, lexeme_end_p lb);
    eprintf "Lexical error: %s@." s;
    exit 1
  | Parser.Error ->
    close_in c;
    reportPos  (lexeme_start_p lb, lexeme_end_p lb);
    eprintf "Syntax error@.";
    exit 1
  | e ->
    close_in c;
    eprintf "Anomaly: %s@." (Printexc.to_string e);
    exit 2
